package jalview.ws.datamodel.alphafold;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import jalview.datamodel.ContactListI;
import jalview.datamodel.ContactListImpl;
import jalview.datamodel.ContactListProviderI;
import jalview.datamodel.ContactMatrixI;
import jalview.datamodel.SequenceI;
import jalview.util.MapUtils;

public class PAEContactMatrix implements ContactMatrixI
{

  SequenceI refSeq = null;

  /**
   * the length that refSeq is expected to be (excluding gaps, of course)
   */
  int length;

  int maxrow = 0, maxcol = 0;

  int[] indices1, indices2;

  float[][] elements;

  float maxscore;

  @SuppressWarnings("unchecked")
  public PAEContactMatrix(SequenceI _refSeq, Map<String, Object> pae_obj)
  {
    refSeq = _refSeq;
    while (refSeq.getDatasetSequence() != null)
    {
      refSeq = refSeq.getDatasetSequence();
    }
    // convert the lists to primitive arrays and store
    length = _refSeq.getEnd() - _refSeq.getStart() + 1;

    if (!MapUtils.containsAKey(pae_obj, "predicted_aligned_error", "pae"))
    {
      parse_version_1_pAE(pae_obj);
      return;
    }
    else
    {
      parse_version_2_pAE(pae_obj);
    }
  }

  /**
   * parse a sane JSON representation of the pAE
   * 
   * @param pae_obj
   */
  private void parse_version_2_pAE(Map<String, Object> pae_obj)
  {
    elements = new float[length][length];
    // this is never going to be reached by the integer rounding.. or is it ?
    maxscore = ((Double) MapUtils.getFirst(pae_obj,
            "max_predicted_aligned_error", "max_pae")).floatValue();
    Iterator<List<Long>> scoreRows = ((List<List<Long>>) MapUtils
            .getFirst(pae_obj, "predicted_aligned_error", "pae"))
            .iterator();
    int row = 0, col = 0;
    while (scoreRows.hasNext())
    {
      Iterator<Long> scores = scoreRows.next().iterator();
      while (scores.hasNext())
      {
        Object d = scores.next();
        if (d instanceof Double)
          elements[row][col++] = ((Double) d).longValue();
        else
          elements[row][col++] = (float) d;
      }
      row++;
      col = 0;
    }
    maxcol = length;
    maxrow = length;
  }

  /**
   * v1 format got ditched 28th July 2022 see
   * https://alphafold.ebi.ac.uk/faq#:~:text=We%20updated%20the%20PAE%20JSON%20file%20format%20on%2028th%20July%202022
   * 
   * @param pae_obj
   */
  private void parse_version_1_pAE(Map<String, Object> pae_obj)
  {
    // assume indices are with respect to range defined by _refSeq on the
    // dataset refSeq
    Iterator<Long> rows = ((List<Long>) pae_obj.get("residue1")).iterator();
    Iterator<Long> cols = ((List<Long>) pae_obj.get("residue2")).iterator();
    Iterator<Double> scores = ((List<Double>) pae_obj.get("distance"))
            .iterator();

    elements = new float[length][length];
    while (scores.hasNext())
    {
      float escore = scores.next().floatValue();
      int row = rows.next().intValue();
      int col = cols.next().intValue();
      if (maxrow < row)
      {
        maxrow = row;
      }
      if (maxcol < col)
      {
        maxcol = col;
      }
      elements[row - 1][col - 1] = escore;
    }

    maxscore = ((Double) MapUtils.getFirst(pae_obj,
            "max_predicted_aligned_error", "max_pae")).floatValue();
  }

  @Override
  public ContactListI getContactList(final int _column)
  {
    if (_column < 0 || _column >= elements.length)
    {
      return null;
    }

    return new ContactListImpl(new ContactListProviderI()
    {
      @Override
      public int getPosition()
      {
        return _column;
      }

      @Override
      public int getContactHeight()
      {
        return maxcol - 1;
      }

      @Override
      public double getContactAt(int column)
      {
        if (column < 0 || column >= elements[_column].length)
        {
          return -1;
        }
        // TODO Auto-generated method stub
        return elements[_column][column];
      }
    });
  }

  @Override
  public float getMin()
  {
    return 0;
  }

  @Override
  public float getMax()
  {
    return maxscore;
  }

  @Override
  public boolean hasReferenceSeq()
  {
    return (refSeq != null);
  }

  @Override
  public SequenceI getReferenceSeq()
  {
    return refSeq;
  }

  @Override
  public String getAnnotDescr()
  {
    return "Predicted Alignment Error for " + refSeq.getName();
  }

  @Override
  public String getAnnotLabel()
  {
    return "pAE Matrix";
  }
}
